#ifndef _GOSTC_HASH_H_
#define _GOSTC_HASH_H_

#include "gostc_common.h"


typedef struct gostc_hash_ctx_t {
	uint32_t H[8];
	uint8_t R[32];
	uint32_t L[8];
	uint32_t SIGMA[8];
	int count;
	uint32_t *longUZ;
	uint8_t hmackey[32];
} gostc_hash_ctx_t;



#ifdef __cplusplus
extern "C" {
#endif

/* Normal hash functions */
extern void* gostc_hash_open(void); /* creates context */
extern void gostc_hash_do(void* ctx, const void* b, int l); /* hashes data */
extern void gostc_hash_close(void * hHash,void* hash); 
/* destroys ctx (anyway), calculates final hash */

/* Hash functions for openssl */
extern void gostc_hash_clear(gostc_hash_ctx_t* ctx); 
/* Clears context w/o freeing. Ctx is invalid, and gostc_hash_reset should be called 
 * to start new hash. */

extern void gostc_hash_reset(gostc_hash_ctx_t* ctx);
/* Clears existing context. Sets cproset UZ.  */

extern void gostc_hash_close_plain(gostc_hash_ctx_t* ctx, void* hash);
/* Finalizes and calculates hash. Ctx is cleared and invalid, NOT freed. 
 * To start new hash - gostc_hash_reset should be called.
 */


/* HMAC functions */
extern void* gostc_hmac_open(void* key, int keylen);
extern void gostc_hmac_do(void* hhandle, void* data, int len);
extern void gostc_hmac_close(void* hhandle, void* hmac);


/* HASH and HMAC 2012 context */

typedef struct gostc_hash12_ctx_t {
	int to_clear;
	uint32_t mode256;
	uint32_t tailSZ;
	uint32_t H[16];
	uint32_t Eps[16];
	uint32_t N[16];
	uint32_t M[16];
	uint64_t K[14][8];
	uint64_t l[8];
	uint64_t pr[8];
/*	uint64_t mul_table [8][256]; */
	uint8_t hmackey[64];
} gostc_hash12_ctx_t;

#define gostc_hmac12_ctx_t gostc_hash12_ctx_t


/* Hash 2012 functions */

extern gostc_hash12_ctx_t* gostc_hash12_new(void);
/* Just allocates memory without cleaning.
 * If context is already allocated - no need to call this function 
 */

extern int gostc_hash12_init(gostc_hash12_ctx_t* ctx, int mode256);
/* Initializes the context and sets mode: 0 - 512 bit, 1 - 256 bit 
 * ret: 0-ok,>0-err,
 */

extern int gostc_hash12_do(gostc_hash12_ctx_t* ctx, void* buf, unsigned int size);
/* Feeds data to hash. May be called multiple times.
 * ret: 0-ok,>0-err,
 */


extern int gostc_hash12_finish(gostc_hash12_ctx_t* ctx, void* hash);
/* Finishes calculation and returns calculated hash (512-bit or 256-bit)
 * The context becomes invalid. 
 * To calculate next hash - gostc_hash12_init should be called.
 * ret: 0-ok,>0-err,
 * If error - the context still may be invalid. 
 */

extern void gostc_hash12_clear(gostc_hash12_ctx_t* ctx);
/* Safely clears the context */

extern void gostc_hash12_free(gostc_hash12_ctx_t** pctx);
/* Clears (if needed) and frees memory and clears the pointer */

int gostc_hash12_test(void);
/* ret: 0-ok,>0-err,
 * Compile with -DTESTOUT to enable diagnostics 
 */



/* HMAC 2012 functions */

#define gostc_hmac12_new gostc_hash12_new
/* Just allocates memory for context. Same as for hash */

extern int gostc_hmac12_init(gostc_hmac12_ctx_t* ctx, void* key, unsigned int keylen);
/* Initializes the context and sets the HMAC key.
 * Only 512-bit mode is implemented.
 * 0-ok, >0-err
 */

#define gostc_hmac12_do gostc_hash12_do
/* Feeds data to hmac. May be called multiple times.
 * Ret: 0-ok, >0-err
 */

extern int gostc_hmac12_finish(gostc_hmac12_ctx_t* ctx, void* hmac);
/* Finishes HMAC calculation and stores 512-bit value to hmac.
 * The context becomes invalid.
 * To calculate next hmac - gostc_hmac12_init should be called.
 * Ret: 0-ok, >0-err
 * If error - the context still may be invalid. 
 */

#define gostc_hmac12_clear gostc_hash12_clear
/* Safely clears the context. Same as for hash */

#define gostc_hmac12_free gostc_hash12_free
/* Just frees memory and clear the pointer. Same as for hash. */


#ifdef __cplusplus
}
#endif



#endif

